@@ -511,12 +511,19 @@ void unlock_comm_sd(void)
 void send_fakem(const struct fake_msg *buf)
 {
   int r;
+  struct fake_msg_buf fm = { 0 };
 
   if(init_get_msg()!=-1){
-    ((struct fake_msg *)buf)->mtype=1;
+    memcpy(&fm.msg, buf, sizeof(*buf));
+    fm.mtype=1;
+#if __BYTE_ORDER == __BIG_ENDIAN
+    ((struct fake_msg*)&fm.msg)->magic=FAKEROOT_MAGIC_BE;
+#elif __BYTE_ORDER == __LITTLE_ENDIAN
+    ((struct fake_msg*)&fm.msg)->magic=FAKEROOT_MAGIC_LE;
+#endif
     do
-      r=msgsnd(msg_snd, (struct fake_msg *)buf,
-	       sizeof(*buf)-sizeof(buf->mtype), 0);
+      r=msgsnd(msg_snd, &fm,
+	       sizeof(fm)-sizeof(fm.mtype), 0);
     while((r==-1) && (errno==EINTR));
     if(r==-1)
       perror("libfakeroot, when sending message");
@@ -548,8 +555,12 @@ void send_get_fakem(struct fake_msg *buf)
   there will always be some (small) chance it will go wrong.
   */
 
+  struct fake_msg_buf fm = { 0 };
+  uint32_t k = 0;
+  uint32_t magic_candidate = 0;
   int l;
   pid_t pid;
+  uint8_t* ptr = NULL;
   static int serial=0;
 
   if(init_get_msg()!=-1){
@@ -560,11 +571,50 @@ void send_get_fakem(struct fake_msg *buf)
     buf->pid=pid;
     send_fakem(buf);
 
-    do
+    do {
       l=msgrcv(msg_get,
-               (struct my_msgbuf*)buf,
-               sizeof(*buf)-sizeof(buf->mtype),0,0);
-    while(((l==-1)&&(errno==EINTR))||(buf->serial!=serial)||buf->pid!=pid);
+               &fm,
+               sizeof(fm)-sizeof(fm.mtype),0,0);
+
+      ptr = &fm;
+      for (k=0; k<16; k++) {
+        magic_candidate = *(uint32_t*)&ptr[k];
+        if (magic_candidate == FAKEROOT_MAGIC_LE || magic_candidate == FAKEROOT_MAGIC_BE) {
+          memcpy(buf, &ptr[k], sizeof(*buf));
+          break;
+        }
+      }
+
+      if (k == 16) {
+        fprintf(stderr,
+               "libfakeroot internal error: payload not recognized!\n");
+        continue;
+      }
+
+      /*
+        Use swapX here instead of ntoh/hton
+        that do nothing on big-endian machines
+      */
+#if __BYTE_ORDER == __LITTLE_ENDIAN
+      if (magic_candidate == FAKEROOT_MAGIC_BE) {
+#elif __BYTE_ORDER == __BIG_ENDIAN
+      if (magic_candidate == FAKEROOT_MAGIC_LE) {
+#endif
+         buf->id = bswapl(buf->id);
+         buf->pid = bswapl(buf->pid);
+         buf->serial = bswapl(buf->serial);
+         buf->st.uid = bswapl(buf->st.uid);
+         buf->st.gid = bswapl(buf->st.gid);
+         buf->st.ino = bswapll(buf->st.ino);
+         buf->st.dev = bswapll(buf->st.dev);
+         buf->st.rdev = bswapll(buf->st.rdev);
+         buf->st.mode = bswapl(buf->st.mode);
+         buf->st.nlink = bswapl(buf->st.nlink);
+         buf->remote = bswapl(0);
+         buf->xattr.buffersize = bswapl(buf->xattr.buffersize);
+         buf->xattr.flags_rc = bswapl(buf->xattr.flags_rc);
+      }
+    }while(((l==-1)&&(errno==EINTR))||(buf->serial!=serial)||buf->pid!=pid);
 
     if(l==-1){
       buf->xattr.flags_rc=errno;
@@ -581,7 +631,6 @@ void send_get_fakem(struct fake_msg *buf)
     printf("libfakeroot/fakeroot, internal bug!! get_fake: length=%i != l=%i",
     sizeof(*buf)-sizeof(buf->mtype),l);
     */
-
   }
 }
 
